import { defu } from 'defu';
import { defineNuxtModule, createResolver, addImportsDir, addPlugin } from '@nuxt/kit';
import { existsSync } from 'node:fs';

const name = "@hexdigital/nuxt-segment";
const version = "1.2.0";

const DEVTOOLS_UI_ROUTE = "/__nuxt-segment";
const DEVTOOLS_UI_LOCAL_PORT = 3300;
function setupDevToolsUI(nuxt, resolver) {
  const clientPath = resolver.resolve("./client");
  const isProductionBuild = existsSync(clientPath);
  if (isProductionBuild) {
    nuxt.hook("vite:serverCreated", async (server) => {
      const sirv = await import('sirv').then((r) => r.default || r);
      server.middlewares.use(
        DEVTOOLS_UI_ROUTE,
        sirv(clientPath, { dev: true, single: true })
      );
    });
  } else {
    nuxt.hook("vite:extendConfig", (config) => {
      config.server = config.server || {};
      config.server.proxy = config.server.proxy || {};
      config.server.proxy[DEVTOOLS_UI_ROUTE] = {
        target: "http://localhost:" + DEVTOOLS_UI_LOCAL_PORT + DEVTOOLS_UI_ROUTE,
        changeOrigin: true,
        followRedirects: true,
        rewrite: (path) => path.replace(DEVTOOLS_UI_ROUTE, "")
      };
    });
  }
  nuxt.hook("devtools:customTabs", (tabs) => {
    tabs.push({
      // unique identifier
      name: "nuxt-segment",
      // title to display in the tab
      title: "Nuxt Segment",
      // any icon from Iconify, or a URL to an image
      icon: "carbon:text-link-analysis",
      // iframe view
      view: {
        type: "iframe",
        src: DEVTOOLS_UI_ROUTE
      }
    });
  });
}

const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: "segment",
    compatibility: {
      nuxt: "^3"
    }
  },
  defaults: {
    debugEnabled: false,
    devtools: true,
    enableRouterSync: true,
    trackDevMode: false,
    ignoredViews: [],
    trackOnNextTick: false,
    settings: {
      writeKey: ""
    },
    options: {}
  },
  setup(options, nuxt) {
    const resolver = createResolver(import.meta.url);
    const moduleOptions = defu(
      nuxt.options.runtimeConfig.public.segment,
      options
    );
    nuxt.options.runtimeConfig.public.segment = moduleOptions;
    nuxt.options.vite.optimizeDeps = nuxt.options.vite.optimizeDeps ?? {};
    nuxt.options.vite.optimizeDeps.include = nuxt.options.vite?.optimizeDeps?.include ?? [];
    nuxt.options.vite.optimizeDeps.include.push("@segment/analytics-next");
    nuxt.options.build.transpile.push(resolver.resolve("./runtime"));
    addImportsDir(resolver.resolve("./runtime/composables"));
    addPlugin({
      src: resolver.resolve("./runtime/plugin.client"),
      mode: "client"
    });
    if (options.devtools) {
      setupDevToolsUI(nuxt, resolver);
    }
  }
});

export { module as default };
